/*----------------------------------------------------------------------------
 Copyright:      
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        20.01.2013
 Description:  

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.   
------------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>

#include "rf22.h"
#include "usart.h"
#include "pca9685.h"

#define TRANSMIT_BLOCKS 16	//1 Block 32Channels max.16 Blocks 512Channels
#define TRANSMIT_REPEAT	64
#define DMX_LOST_TIMEOUT 500

volatile unsigned char dmx_buffer[514];
volatile unsigned char dmx_tmp_buffer[33];
volatile unsigned char new_dmx_data = 0;
volatile unsigned int dmx_lost = DMX_LOST_TIMEOUT;

const unsigned char HIC[16] = {16,15,14,13,9,10,11,12,8,7,6,5,4,3,2,1};

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (TIMER2_COMPA_vect)
//############################################################################
{
	if(dmx_lost<DMX_LOST_TIMEOUT)
	{
		dmx_lost++;
	}
}

//############################################################################
//Hauptprogramm DMX FUNK BERTRAGUNG
int main(void)
//############################################################################
{	
	unsigned int tmp;
	unsigned char packet_cnt = 0;
	unsigned long counter = 0;
	unsigned char addr = 0;
	
	//DEBUG
	usart_init(9600);
	
	pca9685_init(PCA9685_ADDR, PCA9685_FREQUENCY(1200UL));

	
	DDRB |= (1<<PB0); //LED PORT Output
	PORTB |= (1<<0); //LED ON
	for(unsigned long b = 0;b<100000;b++){;asm("nop");};
	
	for(unsigned char led = 0; led < 16; led++) {
		pca9685_pwm(PCA9685_ADDR, led, 20);
	}
	
	//DEBUG
	usart_write("Test\r\n");		
	
	rf22_init();
	rf22_setfreq(RF22FREQ(433.9));
	rf22_rxmode();	
		
	//Timer2 Timecounter fr DMX Ausfall Funk
	TCCR2A |= (1<<WGM21);
	TCCR2B |= (1<<CS22|1<<CS21|1<<CS20);
	TIMSK2 |= (1<<OCIE2A);	
	OCR2A = F_CPU/1024/1000 - 1; //Tick Time
	
	sei();//Globale Interrupts Enable
		
	while(1)
	{	
		//Warte bis Packet empfangen wurde
		while(PIND&(1<<2))
		{	
			if(dmx_lost == DMX_LOST_TIMEOUT)
			{
				PORTB &= ~(1<<0); //Status LED aus
				counter++;
				if(counter >1000000)
				{
					PORTB |= (1<<0);
					for(unsigned long a = 0;a<5000;a++){;asm("nop");};
					PORTB &= ~(1<<0);
					counter = 0;
				}
			}
			else
			{
				PORTB |= (1<<0); //Status LED an
				counter = 0;
			}
		}

		tmp = rf22_getpacket(dmx_tmp_buffer); //Packetcounter und 32DMX Bytes
		
		//wurden alle Bytes empfangen?
		if(tmp == 33)
		{
			dmx_lost= 0;
			//512 Kanle werden zu je 32Bytes a 16 Packete empfangen
			packet_cnt = dmx_tmp_buffer[0];
			memcpy((unsigned char*)&dmx_buffer[(packet_cnt*32)],(unsigned char *)&dmx_tmp_buffer[1],32);
		}
		
		addr = 0;
		if(!(PINC&(1<<PC0))) addr+=1;
		if(!(PINC&(1<<PC1))) addr+=8;
		if(!(PINC&(1<<PC2))) addr+=4;
		if(!(PINC&(1<<PC3))) addr+=2;
		
		for(unsigned char led = 0; led < 16; led++) {
			pca9685_pwm(PCA9685_ADDR, led,dmx_buffer[((addr * 12)+HIC[led])]);
		}	
	}
}
